import inspect
import json
from datetime import date, datetime


class ObjectBaseMeta(type):
    def __call__(cls, *args, **kwargs):
        if len(args) == 1:
            if isinstance(args[0], dict):
                kwargs = {**args[0]}
                args = ()
            elif isinstance(args[0], str) or isinstance(args[0], bytes):
                data = json.loads(args[0])
                if type(data) == list:
                    return cls.__call__(data)
                kwargs = {**data}
                args = ()
            elif isinstance(args[0], list):
                object_list = []
                for data in args[0]:
                    object_list.append(cls.__call__(**data))
                return object_list

        try:
            object_ = object.__new__(cls)
        except:
            object_ = cls.__new__(cls)
        object_.__init__(*args, **kwargs)
        object_.extended_objects = []
        return object_


class ObjectBase:
    @classmethod
    def from_json(cls, json):
        obj: cls = cls.__call__(json)
        return obj

    def to_json(self):
        return json.dumps(dict(self))

    def convert_to_raw_json(self, var):
        if isinstance(var, ObjectBase):
            return str(var)
        if isinstance(var, list):
            return [self.convert_to_raw_json(x) for x in var]
        if isinstance(var, ObjectBase):
            return dict(var)
        if isinstance(var, (date, datetime)):
            return var.isoformat()
        return var

    def __iter__(self):
        for field in list(inspect.signature(self.__init__).parameters)[:-1]:
            yield field, self.convert_to_raw_json(getattr(self, field))

    def __repr__(self):
        return f'<{self.__class__.__name__} {" ".join(x[0] + "=" + str(x[1]) for x in list(self))}>'


class Object(ObjectBase, metaclass = ObjectBaseMeta):
    pass



import contextlib
import copy
import datetime
import inspect
import typing
from enum import Enum
from typing import Callable, Iterable, Sequence, Type, TypeVar

T = TypeVar("T")


def convert(var, type_: Type[T]) -> T:
    if isinstance(type_, type) and isinstance(var, type_):
        return var
    if isinstance(type_, datetime.date) and isinstance(var, str):
        return type_.fromisoformat(var)
    with contextlib.suppress(Exception):
        return type_(var)
    return var


class NamedException:
    def __new__(cls, name, err=None):
        def __init__(self, error):
            super(Exception, self).__init__(error)

        new_class = type(name, (Exception,), {'__init__': __init__})
        return new_class(err) if err else new_class


def TypedListConverter(type_to_convert: Type[T]) -> Callable[[Iterable], typing.List[T]]:
    def _convert(list_to_convert):
        dl = []
        for elm in list_to_convert:
            dl.append(convert(elm, type_to_convert))
        return dl

    return _convert


class List(typing.List):
    def __class_getitem__(cls, item):
        return TypedListConverter(item)


class Union:
    def __class_getitem__(cls, types):
        types = (types,) if not isinstance(types, tuple) else types

        def converter(item):
            dl = []
            original = copy.deepcopy(item)
            if isinstance(item, Sequence):
                for elm in item:
                    original = copy.deepcopy(elm)
                    for type in types:
                        elm = convert(elm, type)
                        if elm is not original:
                            break
                    dl.append(elm)
            else:
                for type in types:
                    item = convert(item, type)
                    if item is not original:
                        break
                dl = item
            return dl

        return converter


def contains(self, value):
    return value in self.__members__


Enum.__class__.__contains__ = contains


def Extras(extras):
    if extras:
        frame = inspect.currentframe().f_back.f_back
        other = frame.f_locals.get('self')
        if not other:
            print(f'Got extra kwarg(s) {extras} in unknown object -> {inspect.getframeinfo(frame)}')
        else:
            name = other.__class__.__name__
            print(f'Got extra kwarg(s) {extras} in {name}')
    return extras


def get_name(object, object_class, object_enum):
    if isinstance(object, object_class):
        object_name = object.name
    elif isinstance(object, object_enum):
        if isinstance(object.value, object_class):
            object_name = object.value.name
        else:
            object_name = object.value
    elif isinstance(object, str) and object.upper() in object_enum:
        object_name = object.upper()
    else:
        raise ValueError(f"Expected a {object_class.__name__}, got {object}")
    return object_name



from __future__ import annotations

import random
from enum import Enum
from typing import Optional, Union

CLIENT_ROVERS = []


class Rover:
    def __init__(self, name: str, client: Client = None):
        self.name: str = name
        self._client = client
        if self._client:
            CLIENT_ROVERS.append(self)

        self.CAMERAS = make_cameras(self)

        self.edl_rucam: CameraClient = self.CAMERAS.EDL_RUCAM.value
        self.edl_rdcam: CameraClient = self.CAMERAS.EDL_RDCAM.value
        self.edl_ddcam: CameraClient = self.CAMERAS.EDL_DDCAM.value
        self.edl_pucam1: CameraClient = self.CAMERAS.EDL_PUCAM1.value
        self.edl_pucam2: CameraClient = self.CAMERAS.EDL_PUCAM2.value
        self.navcam_left: CameraClient = self.CAMERAS.NAVCAM_LEFT.value
        self.navcam_right: CameraClient = self.CAMERAS.NAVCAM_RIGHT.value
        self.mcz_right: CameraClient = self.CAMERAS.MCZ_RIGHT.value
        self.mcz_left: CameraClient = self.CAMERAS.MCZ_LEFT.value
        self.front_hazcam_left_a: CameraClient = self.CAMERAS.FRONT_HAZCAM_LEFT_A.value
        self.front_hazcam_right_a: CameraClient = self.CAMERAS.FRONT_HAZCAM_RIGHT_A.value
        self.rear_hazcam_left: CameraClient = self.CAMERAS.REAR_HAZCAM_LEFT.value
        self.rear_hazcam_right: CameraClient = self.CAMERAS.REAR_HAZCAM_RIGHT.value
        self.skycam: CameraClient = self.CAMERAS.SKYCAM.value
        self.sherloc_watson: CameraClient = self.CAMERAS.SHERLOC_WATSON.value
        self.fhaz: CameraClient = self.CAMERAS.FHAZ.value
        self.rhaz: CameraClient = self.CAMERAS.RHAZ.value
        self.mast: CameraClient = self.CAMERAS.MAST.value
        self.chemcam: CameraClient = self.CAMERAS.CHEMCAM.value
        self.mahli: CameraClient = self.CAMERAS.MAHLI.value
        self.mardi: CameraClient = self.CAMERAS.MARDI.value
        self.navcam: CameraClient = self.CAMERAS.NAVCAM.value
        self.pancam: CameraClient = self.CAMERAS.PANCAM.value
        self.minites: CameraClient = self.CAMERAS.MINITES.value

    def add_client(self, client):
        if not isinstance(client, Client):
            raise TypeError(f"Expected Client, got {client}")
        if self._client and self in CLIENT_ROVERS:
            CLIENT_ROVERS.remove(self)
        return self.__class__(self.name, client)

    @property
    def client(self) -> Client:
        if self._client:
            return self._client
        if CLIENTS:
            return random.choice(list(CLIENTS.values()))
        raise ValueError('No Clients Available!!')

    def get_photos(self, sol: int = None, earth_date: str = None, page_number: Optional[int] = 1,
                   camera: Union[BaseCamera, CAMERAS, str] = None):
        return self.client.get_photos(self, sol, earth_date, page_number, camera)

    def get_photos_by_sol(self, sol: int = None, page_number: Optional[int] = 1,
                          camera: Union[BaseCamera, CAMERAS, str] = None):
        return self.get_photos(sol, None, page_number, camera)

    def get_photos_by_earth_date(self, earth_date: str = None, page_number: Optional[int] = 1,
                                 camera: Union[BaseCamera, CAMERAS, str] = None):
        return self.get_photos(None, earth_date, page_number, camera)

    def get_all_photos(self, sol: int = None, earth_date: str = None, camera: Union[BaseCamera, CAMERAS, str] = None):
        return self.get_photos(sol, earth_date, None, camera)

    def get_all_photos_by_sol(self, sol: int = None, camera: Union[BaseCamera, CAMERAS, str] = None):
        return self.get_all_photos(sol, None, camera)

    def get_all_photos_by_earth_date(self, earth_date: str = None, camera: Union[BaseCamera, CAMERAS, str] = None):
        return self.get_all_photos(None, earth_date, camera)

    def get_latest_photo(self):
        return self.client.get_latest_photo(self)


def make_rovers(client=None):
    class BaseRovers(Enum):
        PERSEVERANCE = Rover("Perseverance", client)
        CURIOSITY = Rover("Curiosity", client)
        OPPORTUNITY = Rover("Opportunity", client)
        SPIRIT = Rover("Spirit", client)

    return BaseRovers


from marstuff.objects.camera import CameraClient, make_cameras

ROVERS = make_rovers()

from marstuff.client import Client, CLIENTS
from marstuff.objects.camera import BaseCamera, CAMERAS



from __future__ import annotations

import os
from datetime import date

import httpx
from PIL import Image

from marstuff.bases import Object
from marstuff.objects.camera import CAMERAS
from marstuff.utils import convert, Extras, List


class Photo(Object):
    _image = None

    def __init__(self, id=None, sol=None, camera=None, img_src=None, earth_date=None, rover=None, **extras):
        self.id = convert(id, int)
        self.sol = convert(sol, int)
        self.camera = convert(camera, Camera)
        self.img_src = convert(img_src, str)
        self.earth_date = convert(earth_date, date)
        self.rover = convert(rover, Manifest)
        self.extras: dict = convert(extras, Extras)

    @property
    def image(self) -> Image.Image:
        if self._image:
            return self._image
        raw_image = httpx.get(self.img_src)
        self._image = Image.open(raw_image)
        return self._image

    def display(self):
        self.image.show()

    def show(self):
        self.display()

    def save(self, path=None):
        if path is None:
            path = f"NASA_{self.rover.name}_{self.camera.name}_{str(self.earth_date).replace('-', '_')}.png"
        else:
            name = os.path.dirname(path)
            if name:
                os.makedirs(name, exist_ok = True)
        self.image.save(path)


class ManifestPhoto(Object):
    def __init__(self, id=None, sol=None, earth_date=None, total_photos=None, cameras=None, **extras):
        self.id = convert(id, int)
        self.sol = convert(sol, int)
        self.earth_date = convert(earth_date, date)
        self.total_photos = convert(total_photos, int)

        for index, camera in enumerate(cameras):
            if not hasattr(CAMERAS, camera):
                print(f'Unknown camera: {camera}')
            else:
                cameras[index] = getattr(CAMERAS, camera).value
        self.cameras = cameras

        self.extras: dict = convert(extras, Extras)


from marstuff.objects.camera import Camera
from marstuff.objects.manifest import Manifest



from __future__ import annotations

import random
from enum import Enum
from typing import Optional

from marstuff.bases import Object
from marstuff.utils import convert, Extras


class BaseCamera:
    pass


class Camera(Object, BaseCamera):
    def __init__(self, id=None, name=None, rover_id=None, full_name=None, **extras):
        self.id = convert(id, int)
        self.name = convert(name, str)
        self.rover_id = convert(rover_id, int)
        self.full_name = convert(full_name, str)
        self.extras: dict = convert(extras, Extras)


class CameraClient(BaseCamera):
    def __init__(self, name: str, full_name: str, rover: Rover = None):
        self.name = name
        self.full_name = full_name
        self._rover = rover

    @property
    def rover(self) -> Rover:
        if self._rover:
            return self._rover
        if CLIENT_ROVERS:
            return random.choice(CLIENT_ROVERS)
        raise ValueError('No Rovers Available!!')

    def add_rover(self, rover):
        if not isinstance(rover, Rover):
            raise TypeError(f"Expected Client, got {rover}")
        return self.__class__(self.name, self.full_name, rover)

    def get_photos(self, sol: int = None, earth_date: str = None, page_number: Optional[int] = 1):
        return self.rover.get_photos(sol, earth_date, page_number, self)

    def get_photos_by_sol(self, sol: int = None, page_number: Optional[int] = 1):
        return self.get_photos(sol, None, page_number)

    def get_photos_by_earth_date(self, earth_date: str = None, page_number: Optional[int] = 1):
        return self.get_photos(None, earth_date, page_number)

    def get_all_photos(self, sol: int = None, earth_date: str = None):
        return self.get_photos(sol, earth_date, None)

    def get_all_photos_by_sol(self, sol: int = None):
        return self.get_all_photos(sol, None)

    def get_all_photos_by_earth_date(self, earth_date: str = None):
        return self.get_all_photos(None, earth_date)

    def __repr__(self):
        return self.name


def make_cameras(rover=None):
    class BaseCameras(Enum):
        EDL_RUCAM = CameraClient("EDL_RUCAM", "Rover Up-Look Camera", rover)
        EDL_RDCAM = CameraClient("EDL_RDCAM", "Rover Down-Look Camera", rover)
        EDL_DDCAM = CameraClient("EDL_DDCAM", "Descent Stage Down-Look Camera", rover)
        EDL_PUCAM1 = CameraClient("EDL_PUCAM1", "Parachute Up-Look Camera A", rover)
        EDL_PUCAM2 = CameraClient("EDL_PUCAM2", "Parachute Up-Look Camera B", rover)
        NAVCAM_LEFT = CameraClient("NAVCAM_LEFT", "Navigation Camera - Left", rover)
        NAVCAM_RIGHT = CameraClient("NAVCAM_RIGHT", "Navigation Camera - Right", rover)
        MCZ_RIGHT = CameraClient("MCZ_RIGHT", "Mast Camera Zoom - Right", rover)
        MCZ_LEFT = CameraClient("MCZ_LEFT", "Mast Camera Zoom - Left", rover)
        FRONT_HAZCAM_LEFT_A = CameraClient("FRONT_HAZCAM_LEFT_A", "Front Hazard Avoidance Camera - Left",
                                                         rover)
        FRONT_HAZCAM_RIGHT_A = CameraClient("FRONT_HAZCAM_RIGHT_A",
                                                          "Front Hazard Avoidance Camera - Right", rover)
        REAR_HAZCAM_LEFT = CameraClient("REAR_HAZCAM_LEFT", "Rear Hazard Avoidance Camera - Left", rover)
        REAR_HAZCAM_RIGHT = CameraClient("REAR_HAZCAM_RIGHT", "Rear Hazard Avoidance Camera - Right",
                                                       rover)
        SKYCAM = CameraClient("SKYCAM", "MEDA Skycam", rover)
        SHERLOC_WATSON = CameraClient("SHERLOC_WATSON", "SHERLOC WATSON Camera", rover)
        FHAZ = CameraClient("FHAZ", "Front Hazard Avoidance Camera", rover)
        RHAZ = CameraClient("RHAZ", "Rear Hazard Avoidance Camera", rover)
        MAST = CameraClient("MAST", "Mast Camera", rover)
        CHEMCAM = CameraClient("CHEMCAM", "Chemistry and Camera Complex", rover)
        MAHLI = CameraClient("MAHLI", "Mars Hand Lens Imager", rover)
        MARDI = CameraClient("MARDI", "Mars Descent Imager", rover)
        NAVCAM = CameraClient("NAVCAM", "Navigation Camera", rover)
        PANCAM = CameraClient("PANCAM", "Panoramic Camera", rover)
        MINITES = CameraClient("MINITES", "Miniature Thermal Emission Spectrometer (Mini-TES)", rover)

    return BaseCameras


CAMERAS = make_cameras()

from marstuff.objects.rover import CLIENT_ROVERS, Rover



from datetime import date

from marstuff.bases import Object
from marstuff.utils import convert, Extras, List


class Manifest(Object):
    def __init__(self, id=None, name=None, landing_date=None, launch_date=None, status=None, photos=None, max_sol=None, max_date=None, total_photos=None, **extras):
        self.id = convert(id, int)
        self.name = convert(name, str)
        self.landing_date = convert(landing_date, date)
        self.launch_date = convert(launch_date, date)
        self.status = convert(status, str)
        self.photos = convert(photos, List[ManifestPhoto])
        self.max_sol = convert(max_sol, int)
        self.max_date = convert(max_date, date)
        self.total_photos = convert(total_photos, int)
        self.extras: dict = convert(extras, Extras)


from marstuff.objects.photo import ManifestPhoto


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['marstuff', 'marstuff.objects']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=8.3.2,<9.0.0', 'httpx[http2]>=0.19.0,<0.20.0']

setup_kwargs = {
    'name': 'marstuff',
    'version': '0.2.0',
    'description': "A Python Wrapper for NASA's Mars Rover Photos API",
    'long_description': '<p align="center"><img width=\'900\' src="https://mars.nasa.gov/system/content_pages/main_images/374_mars2020-PIA21635.jpg"></p>\n<h1 align=\'center\'>Marstuff</h1>\n<h3 align=\'center\'>API Wrapper for NASA\'s Mars Rover Photos API</h3>\n<p align="center">\n  <a href="https://pypi.python.org/pypi/marstuff/"><img src=\'https://img.shields.io/badge/MADE%20WITH-Python-red?style=for-the-badge\'/></a>\n  <a href="https://pypi.python.org/pypi/marstuff/"><img src=\'https://img.shields.io/pypi/pyversions/marstuff?style=for-the-badge\'/></a>\n  <a href="https://pypi.python.org/pypi/marstuff/"><img src=\'https://img.shields.io/pypi/status/marstuff?style=for-the-badge\'/></a>\n  <a href="https://pypi.python.org/pypi/marstuff/"><img src=\'https://img.shields.io/pypi/l/marstuff?style=for-the-badge\'/></a>\n  <a href="https://pypi.python.org/pypi/marstuff/"><img src=\'https://img.shields.io/pypi/v/marstuff?style=for-the-badge\'/></a>\n</p>\n\n\n## 📝 Description\nThis is a **Work-In-Progress** Python API Wrapper for NASA\'s Mars Rover Photos API\nwhich provides both **Synchronous** and **Asynchronous** support.\n\n## 🔥 Getting Started\n- ### ⬇️ Installation\n  - Via pip\n    ```\n    pip install marstuff\n    ```\n- ### ✨ Getting a NASA Developer Key\n  - Goto https://api.nasa.gov/\n  - Navigate to `Generate API Key` at the top navbar.\n  ![image](https://user-images.githubusercontent.com/90889682/136913433-d1324685-4205-4497-a2c7-0619fb1dd97b.png)\n  - Fill in your details, and tap `Signup`\n  ![image](https://user-images.githubusercontent.com/90889682/136914064-40e4fbfc-e8f3-46b2-b02e-b270c7cd5c09.png)\n  - Copy Your API Key\n  ![image](https://user-images.githubusercontent.com/90889682/136915687-fcfdc223-e85e-41f6-bcbb-4781ef1e97bc.png)\n- ### ⚡ Quickstart\n  - Synchronously getting, Viewing and Saving the `Latest Photo` of NASA\'s `Curiosity` Rover\n    ```py\n    from marstuff import Client # Import the Client class\n    client = Client("Your API Token") # Make a new Client\n    \n    # Get the latest Photo\n    photo = client.get_latest_photo(client.curiosity)\n    # OR\n    photo = client.curiosity.get_latest_photo()\n    \n    # Display the Photo\n    photo.show()\n    # Save the Photo\n    photo.save("Latest photo of Curiosity.png")\n    ```\n  - Asynchronously getting all the photos taken by the `Rear Hazard Avoidance Camera` of NASA\'s `Curiosity` Rover on sol `3259`\n    ```py\n    from marstuff import AsyncClient # Import the AsyncClient class\n    from marstuff.objects.camera import CAMERAS # Import the list of all CAMERAS\n    import asyncio # Import asyncio\n    \n    client = AsyncClient("Your API Token") # Make a new Client\n    \n    # Make a function for running asyncio\n    async def get_photos():\n        # Get the photo by Curiosity on sol 3259 with the RHAZ camera\n        photos = await client.curiosity.get_all_photos_by_sol(3259, CAMERAS.RHAZ)\n        # OR\n        photos = await client.curiosity.rhaz.get_all_photos_by_sol(3259)\n        print(photos)\n    \n    asyncio.run(get_photos())\n    \n- #### 🧠 General Need-to-Know Stuff!!\n  This API currently provides access to 4 NASA Rovers (`Perseverance`, `Curiosity`, `Opportunity`, and `Spirit`)\n  Each rover, takes photos of the surface of mars via different cameras\n  \n  - The cameras of the `Perseverance` Rover are\n  \n    Abbreviation | Camera                       \n    ------------ | ------------------------------\n    EDL_RUCAM|Rover Up-Look Camera\n    EDL_RDCAM|Rover Down-Look Camera\n    EDL_DDCAM|Descent Stage Down-Look Camera\n    EDL_PUCAM1|Parachute Up-Look Camera A\n    EDL_PUCAM2|Parachute Up-Look Camera B\n    NAVCAM_LEFT|Navigation Camera - Left\n    NAVCAM_RIGHT|Navigation Camera - Right\n    MCZ_RIGHT|Mast Camera Zoom - Right\n    MCZ_LEFT|Mast Camera Zoom - Left\n    FRONT_HAZCAM_LEFT_A|Front Hazard Avoidance Camera - Left\n    FRONT_HAZCAM_RIGHT_A|Front Hazard Avoidance Camera - Right\n    REAR_HAZCAM_LEFT|Rear Hazard Avoidance Camera - Left\n    REAR_HAZCAM_RIGHT|Rear Hazard Avoidance Camera - Right\n    SKYCAM|MEDA Skycam\n    SHERLOC_WATSON|SHERLOC WATSON Camera\n    \n  - Cameras of other Rovers are\n  \n    Abbreviation | Camera                         | Curiosity | Opportunity | Spirit\n    ------------ | ------------------------------ | --------  | ----------- | ------ |\n    FHAZ|Front Hazard Avoidance Camera|✔|✔|✔|\n    RHAZ|Rear Hazard Avoidance Camera|✔|✔|✔|\n    MAST|Mast Camera| ✔||\n    CHEMCAM|Chemistry and Camera Complex  |✔||\n    MAHLI|Mars Hand Lens Imager|✔||\n    MARDI|Mars Descent Imager|✔||\n    NAVCAM|Navigation Camera|✔|✔|✔|\n    PANCAM|Panoramic Camera| |✔|✔|\n    MINITES|Miniature Thermal Emission Spectrometer (Mini-TES)| |✔|✔|\n    \n  You can query via `sol` or `earth_date`\n  - `sol` means `Martian rotation or day` which can be (0 to `Current Sol of Rover`)\n  - `earth_date` is in the format of `YYYY-MM-DD`\n',
    'author': 'Ajay Ratnam',
    'author_email': 'ajratnam.dev@gmail.com',
    'maintainer': 'Ajay Ratnam',
    'maintainer_email': 'ajratnam.dev@gmail.com',
    'url': 'https://github.com/ajratnam/marstuff',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

from datetime import date
from typing import Optional, Union

import httpx
from marstuff.utils import convert, get_name, List

CLIENTS = {}


class Client:
    def __init__(self, api_key: str = "DEMO_KEY", base_url: str = "https://api.nasa.gov/mars-photos/api/v1/"):
        self.api_key = api_key
        self.base_url = base_url
        CLIENTS[api_key] = self

        self.ROVERS = make_rovers(self)

        self.perseverance: Rover = self.ROVERS.PERSEVERANCE.value
        self.curiosity: Rover = self.ROVERS.CURIOSITY.value
        self.opportunity: Rover = self.ROVERS.OPPORTUNITY.value
        self.spirit: Rover = self.ROVERS.SPIRIT.value

    def get(self, endpoint, **params):
        params['api_key'] = self.api_key
        return httpx.get(self.base_url + endpoint, params=params).json()

    @staticmethod
    def get_params(sol, earth_date, page_number, camera):
        params = {}
        if sol is not None:
            sol = convert(sol, int)
            params['sol'] = sol
        elif earth_date is not None:
            earth_date = convert(earth_date, date)
            params['earth_date'] = earth_date.isoformat()
        if page_number is not None:
            page_number = convert(page_number, int)
            params['page'] = page_number
        if camera is not None:
            camera = get_name(camera, BaseCamera, CAMERAS)
            params['camera'] = camera
        return params

    def get_photos(self, rover: Union[Rover, ROVERS, str], sol: int = None, earth_date: str = None,
                   page_number: Optional[int] = 1, camera: Union[BaseCamera, CAMERAS, str] = None):
        rover_name = get_name(rover, Rover, ROVERS)
        params = self.get_params(sol, earth_date, page_number, camera)
        photos = self.get(f"rovers/{rover_name}/photos", **params)
        return convert(photos['photos'], List[Photo])

    def get_photos_by_sol(self, rover: Union[Rover, ROVERS, str], sol: int = None, page_number: Optional[int] = 1,
                          camera: Union[BaseCamera, CAMERAS, str] = None):
        return self.get_photos(rover, sol, None, page_number, camera)

    def get_photos_by_earth_date(self, rover: Union[Rover, ROVERS, str], earth_date: str = None,
                                 page_number: Optional[int] = 1,
                                 camera: Union[BaseCamera, CAMERAS, str] = None):
        return self.get_photos(rover, None, earth_date, page_number, camera)

    def get_all_photos(self, rover: Union[Rover, ROVERS, str], sol: int = None, earth_date: str = None,
                       camera: Union[BaseCamera, CAMERAS, str] = None):
        return self.get_photos(rover, sol, earth_date, None, camera)

    def get_all_photos_by_sol(self, rover: Union[Rover, ROVERS, str], sol: int = None,
                              camera: Union[BaseCamera, CAMERAS, str] = None):
        return self.get_all_photos(rover, sol, None, camera)

    def get_all_photos_by_earth_date(self, rover: Union[Rover, ROVERS, str], earth_date: str = None,
                                     camera: Union[BaseCamera, CAMERAS, str] = None):
        return self.get_all_photos(rover, None, earth_date, camera)

    def get_latest_photo(self, rover: Union[Rover, ROVERS, str]):
        rover_name = get_name(rover, Rover, ROVERS)
        latest_photo = self.get(f'rovers/{rover_name}/latest_photos')
        return convert(latest_photo['latest_photos'][0], Photo)

    def get_latest_manifest(self, rover: Union[Rover, ROVERS, str]):
        rover_name = get_name(rover, Rover, ROVERS)
        manifest_info = self.get(f'manifests/{rover_name}')
        return convert(manifest_info['photo_manifest'], Manifest)


class AsyncClient(Client):
    async def get(self, endpoint, **params):
        params['api_key'] = self.api_key
        async with httpx.AsyncClient() as client:
            return (await client.get(self.base_url + endpoint, params = params)).json()

    async def get_photos(self, rover: Union[Rover, ROVERS, str], sol: int = None, earth_date: str = None,
                         page_number: Optional[int] = 1, camera: Union[BaseCamera, CAMERAS, str] = None):
        rover_name = get_name(rover, Rover, ROVERS)
        params = self.get_params(sol, earth_date, page_number, camera)
        photos = await self.get(f"rovers/{rover_name}/photos", **params)
        return convert(photos['photos'], List[Photo])

    async def get_latest_photo(self, rover: Union[Rover, ROVERS, str]):
        rover_name = get_name(rover, Rover, ROVERS)
        latest_photo = await self.get(f'rovers/{rover_name}/latest_photos')
        return convert(latest_photo['latest_photos'][0], Photo)


from marstuff.objects.camera import BaseCamera, CAMERAS
from marstuff.objects.photo import Photo
from marstuff.objects.rover import make_rovers, Rover, ROVERS
from marstuff.objects.manifest import Manifest
